/*
 *  soapUI, copyright (C) 2006 eviware.com 
 *
 *  soapUI is free software; you can redistribute it and/or modify it under the 
 *  terms of the GNU Lesser General Public License as published by the Free Software Foundation; 
 *  either version 2.1 of the License, or (at your option) any later version.
 *
 *  soapUI is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
 *  even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 *  See the GNU Lesser General Public License for more details at gnu.org.
 */

package com.eviware.soapui.impl.wsdl.actions.iface;

import java.awt.event.ActionEvent;
import java.io.File;

import javax.swing.AbstractAction;
import javax.swing.Action;

import com.eviware.soapui.impl.wsdl.WsdlInterface;
import com.eviware.soapui.impl.wsdl.support.wsdl.CachedWsdlLoader;
import com.eviware.soapui.support.UISupport;

/**
 * Exports the definition (wsdls and xsds) of a WsdlInterface.
 * 
 * @author Ole.Matzura
 */

public class ExportDefinitionAction extends AbstractAction
{
	private final WsdlInterface iface;
   
   public ExportDefinitionAction( WsdlInterface iface )
   {
      super("Export Definition");
		this.iface = iface;
		putValue( Action.SMALL_ICON, UISupport.createImageIcon( "/exportDefinition.gif" ));
      putValue( Action.SHORT_DESCRIPTION, "Exports the entire WSDL and included/imported files to a local directory");
      putValue( Action.ACCELERATOR_KEY, UISupport.getKeyStroke( "menu P" ));
   }

   public void actionPerformed(ActionEvent e)
	{
     	try
		{
			if( exportDefinition( null ) != null )
				UISupport.showInfoMessage( "Definition exported succesfully" );
		}
		catch (Exception e1)
		{
			UISupport.showErrorMessage( e1 );
		}
   }

	public String exportDefinition( String location ) throws Exception
	{
		File folderName =location == null ? UISupport.getFileDialogs().openDirectory( this, "Select output directory", null ) 
				: new File( location );
		
     	if( folderName == null )
     		return null; 
		
     	CachedWsdlLoader loader = null;
     	
     	if( iface.isCached() )
     	{
     		loader = (CachedWsdlLoader) iface.createWsdlLoader();
     	}
     	else
     	{
			loader = new CachedWsdlLoader( iface.getDefinition() );
     	}
     	
		return loader.saveDefinition( folderName.getAbsolutePath() );
	} 
}
