/*
 *  soapUI, copyright (C) 2006 eviware.com 
 *
 *  soapUI is free software; you can redistribute it and/or modify it under the 
 *  terms of the GNU Lesser General Public License as published by the Free Software Foundation; 
 *  either version 2.1 of the License, or (at your option) any later version.
 *
 *  soapUI is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
 *  even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 *  See the GNU Lesser General Public License for more details at gnu.org.
 */

package com.eviware.soapui.impl.wsdl.actions.support;

import java.awt.event.ActionEvent;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.KeyStroke;

import com.eviware.soapui.support.HelpActionMarker;
import com.eviware.soapui.support.Tools;
import com.eviware.soapui.support.UISupport;

/**
 * Shows an online help page
 * 
 * @author Ole.Matzura
 */

public class ShowOnlineHelpAction extends AbstractAction implements HelpActionMarker
{
	private final String url;

	public ShowOnlineHelpAction( String url )
	{
		this( "Online Help", url, UISupport.getKeyStroke( "F1" ) );
	}
	
	public ShowOnlineHelpAction( String title, String url )
	{
		this( title, url, null );
	}
	
	public ShowOnlineHelpAction( String title, String url, KeyStroke accelerator )
   {
      super( title );
		this.url = url;
      putValue( Action.SHORT_DESCRIPTION, "Show online help" );
      if( accelerator != null )
      	putValue( Action.ACCELERATOR_KEY, accelerator );
      
      putValue( Action.SMALL_ICON, UISupport.HELP_ICON );
   }
   
   public void actionPerformed(ActionEvent e)
	{
   	if( url == null )
   		UISupport.showErrorMessage( "Missing help URL" );
   	else
   		Tools.openURL( url );
	}
}
