/*
 *  soapUI, copyright (C) 2006 eviware.com 
 *
 *  soapUI is free software; you can redistribute it and/or modify it under the 
 *  terms of the GNU Lesser General Public License as published by the Free Software Foundation; 
 *  either version 2.1 of the License, or (at your option) any later version.
 *
 *  soapUI is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
 *  even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 *  See the GNU Lesser General Public License for more details at gnu.org.
 */

package com.eviware.soapui.impl.wsdl.panels.mock;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import javax.swing.AbstractAction;
import javax.swing.AbstractListModel;
import javax.swing.Action;
import javax.swing.BorderFactory;
import javax.swing.DefaultListModel;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JProgressBar;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JTextField;
import javax.swing.ListCellRenderer;
import javax.swing.ListModel;
import javax.swing.text.Document;

import com.eviware.soapui.SoapUI;
import com.eviware.soapui.impl.wsdl.actions.support.ShowOnlineHelpAction;
import com.eviware.soapui.impl.wsdl.mock.WsdlMockOperation;
import com.eviware.soapui.impl.wsdl.mock.WsdlMockService;
import com.eviware.soapui.impl.wsdl.support.HelpUrls;
import com.eviware.soapui.model.mock.MockOperation;
import com.eviware.soapui.model.mock.MockResponse;
import com.eviware.soapui.model.mock.MockResult;
import com.eviware.soapui.model.mock.MockRunner;
import com.eviware.soapui.model.mock.MockServiceListener;
import com.eviware.soapui.model.support.MockRunListenerAdapter;
import com.eviware.soapui.support.DocumentListenerAdapter;
import com.eviware.soapui.support.UISupport;
import com.eviware.soapui.support.action.ActionList;
import com.eviware.soapui.support.components.JXToolBar;
import com.eviware.soapui.ui.support.ModelItemDesktopPanel;
import com.jgoodies.forms.builder.ButtonBarBuilder;

public class WsdlMockServiceDesktopPanel extends ModelItemDesktopPanel<WsdlMockService>
{
	private static final String LOG_TITLE = "Request Log";
	private JButton runButton;
	private MockRunner mockRunner;
	private JButton stopButton;
	private JProgressBar progressBar;
	private DefaultListModel logListModel;
	private JList testLogList;
	private JCheckBox enableLogCheckBox;
	private JScrollPane logScrollPane;
	private JList operationList;
	private InternalMockRunListener mockRunListener;
//	private JButton optionsButton;
	private JTextField pathTextField;
	private JTextField portTextField;

	public WsdlMockServiceDesktopPanel( WsdlMockService mockService )
	{
		super( mockService );
		buildUI();
		
		setPreferredSize( new Dimension( 350, 500 ) );
		
		mockRunListener = new InternalMockRunListener();
		mockService.addMockRunListener( mockRunListener  );
	}
	
	public boolean onClose( boolean canCancel )
	{
		if( mockRunner != null && canCancel )
		{
			if( !UISupport.confirm( "Close and stop MockService", "Close MockService" ))
			{
				return false;
			}
		}
		
		if( mockRunner != null )
		{
			mockRunner.stop();
		}
		
		getModelItem().removeMockRunListener( mockRunListener );
		((OperationListModel)operationList.getModel()).release();
		
		return release();
	}

	private void buildUI()
	{
      add( buildToolbar(), BorderLayout.NORTH );
      
      JSplitPane splitPane = UISupport.createVerticalSplit();
		splitPane.setTopComponent( buildOperationList() );
      splitPane.setBottomComponent( buildLog() );
      splitPane.setDividerLocation( 0.7 );
      splitPane.setDividerLocation( 250 );

      add( splitPane, BorderLayout.CENTER );
      add( new JLabel( "--"), BorderLayout.PAGE_END );
	}

   public boolean logIsEnabled()
   {
   	return enableLogCheckBox.isSelected();
   }
   
	private Component buildOperationList()
	{
		JPanel panel = new JPanel( new BorderLayout() );
		ButtonBarBuilder builder = new ButtonBarBuilder();
		
		enableLogCheckBox = new JCheckBox("Enable Log", true);
		enableLogCheckBox.addActionListener( new ActionListener() {

			public void actionPerformed( ActionEvent arg0 )
			{
				testLogList.setEnabled( !enableLogCheckBox.isSelected() );
				
				// border needs to be repainted..
				logScrollPane.repaint();
			}}  );
		
		builder.addFixed( enableLogCheckBox );
		builder.addGlue();
		
		progressBar = new JProgressBar();
      JPanel progressBarPanel = UISupport.createProgressBarPanel( progressBar, 2, false );
      progressBarPanel.setPreferredSize( new Dimension( 60, 20 ) );
      
		builder.addFixed( progressBarPanel);
		builder.setBorder( BorderFactory.createEmptyBorder( 2, 3, 3, 3 ) );

		panel.add( builder.getPanel(), BorderLayout.NORTH );
		
		operationList = new JList( new OperationListModel() );
		operationList.addMouseListener( new ModelItemListMouseListener() );
		operationList.setCellRenderer( new OperationListCellRenderer() );
		
		JScrollPane scrollPane = new JScrollPane( operationList );
		UISupport.addTitledBorder( scrollPane, "Operations" );
		panel.add( scrollPane, BorderLayout.CENTER );
		
		return panel;
	}
	
	private Component buildLog()
	{
		logListModel = new DefaultListModel();
		testLogList = new JList(logListModel);
		testLogList.setCellRenderer(new LogCellRenderer());
		testLogList.addMouseListener(new LogListMouseListener());
		testLogList.setBorder( BorderFactory.createLineBorder(  Color.GRAY ) );

		logScrollPane = new JScrollPane(testLogList);
		logScrollPane.setBorder( null );
		UISupport.addTitledBorder( logScrollPane, LOG_TITLE );
		return logScrollPane;
	}

	private Component buildToolbar()
	{
		JXToolBar toolbar = UISupport.createToolbar();
		
		runButton = createActionButton(new RunMockServiceAction(), true);
		stopButton = createActionButton(new StopMockServiceAction(), false);
//		optionsButton = createActionButton( new MockServiceOptionsAction( getModelItem() ), true );
		
		toolbar.addFixed( runButton );
		toolbar.addFixed( stopButton );
		toolbar.addUnrelatedGap();
		
		pathTextField = new JTextField(15);
		pathTextField.setToolTipText( "The URL path to listen on for this service" );
		pathTextField.setText( getModelItem().getPath() );
		pathTextField.setCaretPosition( 0 );
		pathTextField.getDocument().addDocumentListener( new DocumentListenerAdapter() {

			@Override
			public void update( Document document )
			{
				getModelItem().setPath( pathTextField.getText() );
			}}  );
		
		toolbar.addLabeledFixed( "Path:", pathTextField );
		toolbar.addUnrelatedGap();
		
		portTextField = new JTextField(5);
		portTextField.setToolTipText( "The local port to listen on for this service" );
		portTextField.setText( String.valueOf( getModelItem().getPort() ));
		portTextField.getDocument().addDocumentListener( new DocumentListenerAdapter() {

			@Override
			public void update( Document document )
			{
				try
				{
					getModelItem().setPort( Integer.parseInt( portTextField.getText() ) );
				}
				catch( Exception e )
				{
				}				
			}}  );
		
		toolbar.addLabeledFixed( "Port:", portTextField );
		
		toolbar.addGlue();
	//	toolbar.addFixed( optionsButton );
		toolbar.addFixed( createActionButton( new ShowOnlineHelpAction(HelpUrls.MOCKSERVICE_HELP_URL), true ) );
			
		return toolbar;
	}
	
	private final class InternalMockRunListener extends MockRunListenerAdapter
	{
		public void onMockResult( MockResult result )
		{
			if( logIsEnabled() )
			{
				logListModel.addElement( result );
			}
		}
	}

	public class OperationListModel extends AbstractListModel implements ListModel, MockServiceListener, PropertyChangeListener
	{
		private List<WsdlMockOperation> operations = new ArrayList<WsdlMockOperation>();
		
		public OperationListModel()
		{
			for( int c = 0; c < getModelItem().getMockOperationCount(); c++ )
			{
				WsdlMockOperation mockOperation = ( WsdlMockOperation ) getModelItem().getMockOperationAt( c );
				mockOperation.addPropertyChangeListener( this );
				
				operations.add( mockOperation);
			}
			
			getModelItem().addMockServiceListener( this );
		}
		
		public Object getElementAt( int arg0 )
		{
			return operations.get( arg0 );
		}

		public int getSize()
		{
			return operations.size();
		}

		public void mockOperationAdded( MockOperation operation )
		{
			operations.add( ( WsdlMockOperation ) operation );
			operation.addPropertyChangeListener( this );
			fireIntervalAdded( this, operations.size()-1, operations.size()-1 );
		}

		public void mockOperationRemoved( MockOperation operation )
		{
			int ix = operations.indexOf( operation );
			operations.remove( ix );
			operation.removePropertyChangeListener( this );
			fireIntervalRemoved( this, ix, ix );
		}

		public void mockResponseAdded( MockResponse request )
		{
		}

		public void mockResponseRemoved( MockResponse request )
		{
		}

		public void propertyChange( PropertyChangeEvent arg0 )
		{
			if( arg0.getPropertyName().equals( WsdlMockOperation.NAME_PROPERTY ))
			{
				int ix = operations.indexOf( arg0.getSource() );
				fireContentsChanged( this, ix, ix );
			}
		}
		
		public void release()
		{
			for( WsdlMockOperation operation : operations )
			{
				operation.removePropertyChangeListener( this );
			}
			
			getModelItem().removeMockServiceListener( this );
		}
	}
	
	private final static class OperationListCellRenderer extends JLabel implements ListCellRenderer
	{
		public Component getListCellRendererComponent(JList list, Object value, int index, boolean isSelected,
				boolean cellHasFocus)
		{
			MockOperation testStep = (MockOperation) value;
			setText(testStep.getName());
			setIcon(testStep.getIcon());

			if (isSelected)
			{
				setBackground(list.getSelectionBackground());
				setForeground(list.getSelectionForeground());
			}
			else
			{
				setBackground(list.getBackground());
				setForeground(list.getForeground());
			}

			setEnabled(list.isEnabled());
			setFont(list.getFont());
			setOpaque(true);
			setBorder(BorderFactory.createEmptyBorder(3, 3, 3, 3));

			return this;
		}
	}
	
	public class RunMockServiceAction extends AbstractAction
	{
		public RunMockServiceAction()
		{
			putValue(Action.SMALL_ICON, UISupport.createImageIcon("/submit_request.gif"));
			putValue(Action.SHORT_DESCRIPTION, "Starts this MockService on the specified port and endpoint");
			putValue(Action.ACCELERATOR_KEY, UISupport.getKeyStroke( "alt ENTER" ));
		}
		
		public void actionPerformed( ActionEvent arg0 )
		{
			if( mockRunner != null || SoapUI.getMockEngine().hasRunningMock( getModelItem() ) )
			{
				UISupport.showErrorMessage( "MockService is already running" );
			}
			else
			{
				try
				{
					mockRunner = getModelItem().start();
				}
				catch( Exception e )
				{
					UISupport.showErrorMessage( e );
					e.printStackTrace();
					return;
				}
			}
			
			progressBar.setIndeterminate( true );
			
			runButton.setEnabled( false );
			stopButton.setEnabled( true );
			pathTextField.setEnabled( false );
			portTextField.setEnabled( false );
		}
	}
	
	public class StopMockServiceAction extends AbstractAction
	{
		public StopMockServiceAction()
		{
			putValue(Action.SMALL_ICON, UISupport.createImageIcon("/cancel_request.gif"));
			putValue(Action.SHORT_DESCRIPTION, "Stops this MockService on the specified port and endpoint");
		}
		
		public void actionPerformed( ActionEvent arg0 )
		{
			if( mockRunner == null )
			{
				UISupport.showErrorMessage( "MockService is not running" );
			}
			else
			{
				mockRunner.stop();
				mockRunner = null;
			}
			
			progressBar.setIndeterminate( false );
			
			runButton.setEnabled( true );
			stopButton.setEnabled( false );
			pathTextField.setEnabled( true );
			portTextField.setEnabled( true );
		}
	}
	
	private static final class LogCellRenderer extends JLabel implements ListCellRenderer
	{
		private SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss.SSS");
		
		public Component getListCellRendererComponent(JList list, Object value, int index, boolean isSelected,
				boolean cellHasFocus)
		{
			setIcon(null);

			if (value instanceof String)
			{
				setText(value.toString());
			}
			else if (value instanceof MockResult)
			{
				MockResult result = ( MockResult ) value;
				setIcon(null);
				String msg = dateFormat.format( new Date( result.getTimestamp()) );
				MockResponse mockResponse = result.getMockResponse();
				
				if( mockResponse == null )
				{
					msg += ": [dispatch error; missing response]";
				}
				else
				{
					msg += ": [" + mockResponse.getMockOperation().getName();
					msg += "] " + result.getTimeTaken() + "ms";
				}
				
				setText( msg);
			}

			if (isSelected)
			{
				setBackground(list.getSelectionBackground());
				setForeground(list.getSelectionForeground());
			}
			else
			{
				setBackground(list.getBackground());
				setForeground(list.getForeground());
			}

			setEnabled(list.isEnabled());
			setFont(list.getFont());
			setOpaque(true);
			setBorder(BorderFactory.createEmptyBorder(3, 3, 3, 3));

			return this;
		}
	}
	
	/**
	 * Mouse Listener for triggering default action and showing popup for log list items
	 * 
	 * @author Ole.Matzura
	 */

	private final class LogListMouseListener extends ListMouseListener
	{
		@Override
		protected ActionList getActionsForRow( JList list, int row )
		{
			MockResult result = ( MockResult ) logListModel.getElementAt( row );
			return result == null ? null : result.getActions();
		}
	}
}
