/*
 *  soapUI, copyright (C) 2006 eviware.com 
 *
 *  soapUI is free software; you can redistribute it and/or modify it under the 
 *  terms of the GNU Lesser General Public License as published by the Free Software Foundation; 
 *  either version 2.1 of the License, or (at your option) any later version.
 *
 *  soapUI is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
 *  even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 *  See the GNU Lesser General Public License for more details at gnu.org.
 */

package com.eviware.soapui.impl.wsdl.support.soap;

import java.text.Collator;
import java.util.Collections;
import java.util.Comparator;
import java.util.Iterator;
import java.util.List;

import javax.wsdl.Binding;
import javax.wsdl.BindingOperation;
import javax.wsdl.extensions.soap.SOAPBinding;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.eviware.soapui.impl.wsdl.WsdlInterface;
import com.eviware.soapui.impl.wsdl.WsdlProject;
import com.eviware.soapui.impl.wsdl.support.BindingImporter;
import com.eviware.soapui.impl.wsdl.support.Constants;
import com.eviware.soapui.impl.wsdl.support.wsdl.WsdlContext;
import com.eviware.soapui.impl.wsdl.support.wsdl.WsdlUtils;
import com.eviware.soapui.settings.WsdlSettings;

/**
 * BindingImporter that can import a WsdlInterface from an SOAP/HTTP binding
 * 
 * @author Ole.Matzura
 */

public class Soap11HttpBindingImporter implements BindingImporter
{
   private final static Log log = LogFactory.getLog( Soap11HttpBindingImporter.class );
   
   public boolean canImport(Binding binding)
   {
      List list = binding.getExtensibilityElements();
      SOAPBinding soapBinding = (SOAPBinding) WsdlUtils.getExtensiblityElement( list, SOAPBinding.class );
      return soapBinding == null ? false : 
         soapBinding.getTransportURI().startsWith( Constants.SOAP_HTTP_BINDING_NS );
   }

   public WsdlInterface importBinding( WsdlProject project, WsdlContext wsdlContext, Binding binding ) throws Exception
   {
   	String name = project.getSettings().getBoolean( WsdlSettings.NAME_WITH_BINDING ) ? 
   				binding.getQName().getLocalPart() : binding.getPortType().getQName().getLocalPart();
   	
      WsdlInterface iface = project.addNewInterface( name );
      iface.setBindingName( binding.getQName() );
      iface.setSoapVersion( SoapVersion.Soap11 );
    
      String [] endpoints = WsdlUtils.getEndpointsForBinding( wsdlContext.getDefinition(), binding );
      for (int i = 0; i < endpoints.length; i++)
      {
         log.info("importing endpoint " + endpoints[i]);
         iface.addEndpoint( endpoints[i] );
      }
      
      List<BindingOperation> list = binding.getBindingOperations();
      Collections.sort( list, new BindingOperationComparator());
      
      for (Iterator iter = list.iterator(); iter.hasNext();)
      {
         BindingOperation operation = (BindingOperation) iter.next();
         log.info("importing operation " + operation.getName() );
         iface.addNewOperation( operation );
      }
      
      return iface;
   }
   
   private static final class BindingOperationComparator implements Comparator<BindingOperation>
	{
		public int compare(BindingOperation o1, BindingOperation o2)
		{
		   return Collator.getInstance().compare( o1.getOperation().getName(), o2.getOperation().getName() );
		}
	}
}
