package com.eviware.soapui.impl.wsdl.teststeps;

import com.eviware.soapui.impl.wsdl.WsdlOperation;
import com.eviware.soapui.impl.wsdl.WsdlRequest;
import com.eviware.soapui.impl.wsdl.submit.WsdlMessageExchange;
import com.eviware.soapui.impl.wsdl.submit.transports.http.WsdlResponse;
import com.eviware.soapui.model.iface.Attachment;
import com.eviware.soapui.support.types.StringToStringMap;

public class WsdlResponseMessageExchange extends WsdlMessageExchange
{
	private final WsdlRequest request;
	private WsdlResponse response;
	private String requestContent;

	public WsdlResponseMessageExchange( WsdlRequest request )
	{
		this.request = request;
		response = request.getResponse();
	}
	
	public void setResponse( WsdlResponse response )
	{
		this.response = response;
	}

	public String getRequestContent()
	{
		if( requestContent != null )
			return requestContent;
		
		if( response == null )
			response = request.getResponse();
		
		return response == null ? request.getRequestContent() : response.getRequestContent(); 
	}

	public StringToStringMap getRequestHeaders()
	{
		if( response == null )
			response = request.getResponse();
		
		return response == null ? request.getRequestHeaders() : response.getRequestHeaders(); 
	}
	
	public Attachment[] getRequestAttachments()
	{
		return request.getAttachments();
	}

	public Attachment[] getResponseAttachments()
	{
		if( response == null )
			response = request.getResponse();
		
		return response == null ? null : response.getAttachments();
	}

	public String getResponseContent()
	{
		if( response == null )
			response = request.getResponse();
		
		return response == null ? null : response.getContentAsString();
	}

	public StringToStringMap getResponseHeaders()
	{
		if( response == null )
			response = request.getResponse();
		
		return response == null ? null : response.getResponseHeaders();
	}

	public WsdlOperation getOperation()
	{
		return request.getOperation();
	}

	public long getTimeTaken()
	{
		if( response == null )
			response = request.getResponse();
		
		return response == null ? 0 : response.getTimeTaken();
	}

	public long getTimestamp()
	{
		if( response == null )
			response = request.getResponse();
		
		return response == null ? 0 : response.getTimestamp();
	}

	public void setRequestContent( String requestContent )
	{
		this.requestContent = requestContent;
	}
}
