/*
 *  soapUI, copyright (C) 2006 eviware.com 
 *
 *  soapUI is free software; you can redistribute it and/or modify it under the 
 *  terms of the GNU Lesser General Public License as published by the Free Software Foundation; 
 *  either version 2.1 of the License, or (at your option) any later version.
 *
 *  soapUI is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
 *  even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 *  See the GNU Lesser General Public License for more details at gnu.org.
 */

package com.eviware.soapui.model.support;

import com.eviware.soapui.model.testsuite.TestStep;
import com.eviware.soapui.model.testsuite.TestStepProperty;

/***
 * Default implementation of TestStepProperty interface 
 * 
 * @author Ole.Matzura
 */

public class DefaultTestStepProperty implements TestStepProperty
{
	private final String name;
	private boolean isReadOnly;
	private String description;
	private PropertyHandler handler;
	private final TestStep testStep;
	
	public DefaultTestStepProperty(String name, boolean isReadOnly, PropertyHandler handler, TestStep testStep )
	{
		this.name = name;
		this.isReadOnly = isReadOnly;
		this.handler = handler;
		this.testStep = testStep;
	}
	
	public DefaultTestStepProperty(String name, TestStep testStep)
	{
		this( name, false, new SimplePropertyHandler(), testStep );
	}

	public DefaultTestStepProperty(String name, boolean isReadOnly, TestStep testStep)
	{
		this( name, isReadOnly, new SimplePropertyHandler(), testStep );
	}

	public String getDescription()
	{
		return description;
	}

	public void setDescription(String description)
	{
		this.description = description;
	}

	public String getName()
	{
		return name;
	}

	public void setIsReadOnly( boolean isReadOnly )
	{
		this.isReadOnly = isReadOnly;
	}

	public boolean isReadOnly()
	{
		return isReadOnly;
	}

	public void setPropertyHandler( PropertyHandler handler )
	{
		this.handler = handler;
	}
	
	public String getValue()
	{
		return handler == null ? null : handler.getValue();
	}
	
	public void setValue( String value )
	{
		if( handler != null && !isReadOnly() )
		{
			handler.setValue( value );
		}
	}
	
	public TestStep getTestStep()
	{
		return testStep;
	}

	/**
	 * Handler for providing and setting property values
	 * 
	 * @author Ole.Matzura
	 */
	
	public interface PropertyHandler
	{
		public String getValue();
		
		public void setValue( String value );
	}
	
	/**
	 * Empty implementation of PropertyHandler interface
	 * 
	 * @author Ole.Matzura
	 */
	
	public static class PropertyHandlerAdapter implements PropertyHandler
	{
		public String getValue()
		{
			return null;
		}

		public void setValue(String value)
		{
		}
	}
	
	/**
	 * Simple implementation of PropertyHandler interface
	 * 
	 * @author Ole.Matzura
	 */
	
	public static class SimplePropertyHandler implements PropertyHandler
	{
		private String value;

		public String getValue()
		{
			return value;
		}

		public void setValue(String value)
		{
			this.value = value;
		}}
}
