/*
 *  soapUI, copyright (C) 2006 eviware.com 
 *
 *  soapUI is free software; you can redistribute it and/or modify it under the 
 *  terms of the GNU Lesser General Public License as published by the Free Software Foundation; 
 *  either version 2.1 of the License, or (at your option) any later version.
 *
 *  soapUI is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
 *  even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 *  See the GNU Lesser General Public License for more details at gnu.org.
 */

package com.eviware.soapui.support.action;

import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;

public abstract class AbstractSoapUIAction<T> implements SoapUIAction<T>
{
	private PropertyChangeSupport propertySupport;
	private String name;
	private String description;
	private boolean enabled = true;

	public AbstractSoapUIAction( String name, String description )
	{
		this.name = name;
		this.description = description;
		
		propertySupport = new PropertyChangeSupport( this );
	}
	
	public String getDescription()
	{
		return description;
	}

	public void setEnabled(boolean enabled)
	{
		if( enabled == this.enabled )
			return;
		
		boolean oldEnabled = this.enabled;
		this.enabled = enabled;
		
		propertySupport.firePropertyChange( ENABLED_PROPERTY, oldEnabled, enabled );
	}

	public String getName()
	{
		return name;
	}

	public boolean isEnabled()
	{
		return enabled;
	}

	public abstract void perform(T target);

	public void addPropertyChangeListener(String propertyName, PropertyChangeListener listener)
	{
		propertySupport.addPropertyChangeListener( propertyName, listener );
	}

	public void addPropertyChangeListener(PropertyChangeListener listener)
	{
		propertySupport.addPropertyChangeListener( listener );
	}

	public void removePropertyChangeListener(PropertyChangeListener listener)
	{
		propertySupport.removePropertyChangeListener( listener );
	}

	public void removePropertyChangeListener(String propertyName, PropertyChangeListener listener)
	{
		propertySupport.removePropertyChangeListener( propertyName, listener );
	}
}
