/*
 *  soapUI, copyright (C) 2006 eviware.com 
 *
 *  soapUI is free software; you can redistribute it and/or modify it under the 
 *  terms of the GNU Lesser General Public License as published by the Free Software Foundation; 
 *  either version 2.1 of the License, or (at your option) any later version.
 *
 *  soapUI is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
 *  even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 *  See the GNU Lesser General Public License for more details at gnu.org.
 */

package com.eviware.x.form.support;

import java.lang.reflect.Field;

import com.eviware.soapui.support.UISupport;
import com.eviware.soapui.support.action.ActionList;
import com.eviware.x.form.XForm;
import com.eviware.x.form.XFormDialog;
import com.eviware.x.form.XFormDialogBuilder;
import com.eviware.x.form.XFormFactory;
import com.eviware.x.form.XFormField;
import com.eviware.x.form.XFormTextField;
import com.eviware.x.form.XForm.FieldType;

public class ADialogBuilder
{
	public static XFormDialog buildDialog( Class<? extends Object> formClass )
	{
		return buildDialog( formClass, null );
	}

	public static XFormDialog buildDialog( Class<? extends Object> formClass, ActionList actions )
	{
		AForm formAnnotation = formClass.getAnnotation( AForm.class );
		if( formAnnotation == null )
		{
			throw new RuntimeException( "formClass is not annotated correctly.." );
		}
		
		XFormDialogBuilder builder = XFormFactory.createDialogBuilder(formAnnotation.name());
      XForm form = builder.createForm( "Basic" );
      
      for( Field field : formClass.getFields() )
      {
      	AField fieldAnnotation = field.getAnnotation( AField.class );
      	if( fieldAnnotation != null )
      	{
      		addFormField( form, fieldAnnotation );
      	}
      }
      
      ActionList defaultActions = formAnnotation.helpUrl().length() == 0 ? 
      			builder.buildOkCancelActions() : builder.buildOkCancelHelpActions( formAnnotation.helpUrl() );

      if( actions == null )
      	actions = defaultActions;
      else
      	actions.addActions( defaultActions );
      
      XFormDialog dialog = builder.buildDialog( actions, formAnnotation.description(), 
      			UISupport.createImageIcon( formAnnotation.icon() ));
      
		return dialog;
	}
	
	public static XFormDialog buildTabbedDialog( Class<? extends Object> tabbedFormClass, ActionList actions )
	{
		AForm formAnnotation = tabbedFormClass.getAnnotation( AForm.class );
		if( formAnnotation == null )
		{
			throw new RuntimeException( "formClass is not annotated correctly.." );
		}
		
		XFormDialogBuilder builder = XFormFactory.createDialogBuilder(formAnnotation.name());
		
	   for( Field field : tabbedFormClass.getFields() )
      {
      	AField fieldAnnotation = field.getAnnotation( AField.class );
      	if( fieldAnnotation != null )
      	{
      		XForm form = builder.createForm( fieldAnnotation.name() );
            
      		try
				{
					Class formClass = Class.forName( fieldAnnotation.description() );
					
					for( Field formField : formClass.getFields() )
					{
						AField formFieldAnnotation = formField.getAnnotation( AField.class );
						if( formFieldAnnotation != null )
						{
							addFormField( form, formFieldAnnotation );
						}
					}
				}
				catch( Exception e )
				{
					e.printStackTrace();
				}
      	}
      }
      
      ActionList defaultActions = formAnnotation.helpUrl().length() == 0 ? 
      			builder.buildOkCancelActions() : builder.buildOkCancelHelpActions( formAnnotation.helpUrl() );

      if( actions == null )
      	actions = defaultActions;
      else
      	actions.addActions( defaultActions );
      
      XFormDialog dialog = builder.buildDialog( actions, formAnnotation.description(), 
      			UISupport.createImageIcon( formAnnotation.icon() ));
      
		return dialog;
	}

	private static void addFormField( XForm form, AField fieldAnnotation )
	{
		XFormField field = null;
		
		switch( fieldAnnotation.type() )
		{
			case STRING :
			{
				field = form.addTextField( fieldAnnotation.name(), fieldAnnotation.description(), FieldType.TEXT );
				break;
			}
			case INT :
			{
				field = form.addTextField( fieldAnnotation.name(), fieldAnnotation.description(), FieldType.TEXT );
				((XFormTextField)field).setWidth( 10 );
				break;
			}
			case STRINGAREA :
			{
				field = form.addTextField( fieldAnnotation.name(), fieldAnnotation.description(), FieldType.TEXTAREA );
				break;
			}
			case BOOLEAN :
			{
				field = form.addCheckBox( fieldAnnotation.name(), fieldAnnotation.description() );
				break;
			}
			case FILE :
			{
				field = form.addTextField( fieldAnnotation.name(), fieldAnnotation.description(), FieldType.FILE );
				break;
			}
			case ENUMERATION :
			{
				field = form.addComboBox( fieldAnnotation.name(), fieldAnnotation.values(), fieldAnnotation.description() );
				break;
			}
		}
		
		if( field != null )
			field.setEnabled( fieldAnnotation.enabled() );
	}

}
