/*
 *  soapUI, copyright (C) 2006 eviware.com 
 *
 *  soapUI is free software; you can redistribute it and/or modify it under the 
 *  terms of the GNU Lesser General Public License as published by the Free Software Foundation; 
 *  either version 2.1 of the License, or (at your option) any later version.
 *
 *  soapUI is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
 *  even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 *  See the GNU Lesser General Public License for more details at gnu.org.
 */

package com.eviware.x.impl.swing;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.util.Arrays;

import javax.swing.BorderFactory;
import javax.swing.ImageIcon;
import javax.swing.JDialog;
import javax.swing.JPanel;

import com.eviware.soapui.support.UISupport;
import com.eviware.soapui.support.action.ActionList;
import com.eviware.soapui.support.components.JButtonBar;
import com.eviware.soapui.support.types.StringToStringMap;
import com.eviware.x.form.ValidationMessage;
import com.eviware.x.form.XForm;
import com.eviware.x.form.XFormDialog;
import com.eviware.x.form.XFormField;

public class JFormDialog extends SwingXFormDialog
{
	private JDialog dialog;
	private SwingXFormImpl form;

	public JFormDialog(String name, XForm form, ActionList actions, String description, ImageIcon icon )
	{
		dialog = new JDialog( UISupport.getMainFrame(), name, true );
		
		JButtonBar buttons = UISupport.initDialogActions( actions, dialog );
		buttons.setBorder( BorderFactory.createEmptyBorder( 5, 0, 0, 0 ));
		
		JPanel panel = new JPanel( new BorderLayout() );
		this.form = (SwingXFormImpl)form;
		panel.add( (this.form.getPanel()), BorderLayout.CENTER );
		panel.setBorder( BorderFactory.createEmptyBorder( 5, 5, 5, 5 ));
		
		if( description != null || icon != null )
			dialog.getContentPane().add( UISupport.buildDescription( name, description, icon ), BorderLayout.NORTH );
		
		dialog.getContentPane().add( panel, BorderLayout.CENTER );
		
		buttons.setBorder(
				BorderFactory.createCompoundBorder( 
						BorderFactory.createCompoundBorder( 
								BorderFactory.createMatteBorder( 1, 0, 0, 0, Color.GRAY ),
								BorderFactory.createMatteBorder( 1, 0, 0, 0, Color.WHITE )),
						BorderFactory.createEmptyBorder( 3, 5, 3, 5 )));
		
		dialog.getContentPane().add( buttons, BorderLayout.SOUTH );
	}

	public void setValues(StringToStringMap values)
	{
		form.setValues( values );
	}

	public StringToStringMap getValues()
	{
		StringToStringMap result = new StringToStringMap();
		result.putAll( form.getValues());
		
		return result;
	}

	public void setOptions(String field, Object[] options)
	{
		form.setOptions( field, options );
	}

	public void setVisible(boolean visible)
	{
		dialog.pack();
		Dimension size = dialog.getSize();
		if( size.getHeight() < 270 )
		{
			dialog.setSize( new Dimension( ( int ) size.getWidth(), 270 ) );
		}
		
		UISupport.centerDialog( dialog );
		
		dialog.setVisible( visible );
	}

	public boolean validate()
	{
		XFormField[] formFields = form.getFormFields();
		for( int c = 0; c < formFields.length; c++ )
		{
			ValidationMessage [] messages = formFields[c].validate();
			if( messages != null && messages.length > 0 )
			{
				((AbstractSwingXFormField)messages[0].getFormField()).getComponent().requestFocus();
				UISupport.showErrorMessage( messages[0].getMessage() );
				return false;
			}
		}
		
		return true;
	}

	public void setFormFieldProperty(String name, Object value)
	{
		form.setFormFieldProperty( name, value );
	}

	public String getValue( String field )
	{
		return form.getComponentValue( field );
	}

	public void setValue( String field, String value )
	{
		form.setComponentValue( field, value );
	}

	public int getValueIndex( String name )
	{
		String [] options = form.getOptions( name );
		if( options == null )
			return -1;
		
		return Arrays.asList( options ).indexOf( form.getComponentValue( name ) );
	}

	public boolean show()
	{
		show( new StringToStringMap() );
		return getReturnValue() == XFormDialog.OK_OPTION;
	}

	public XFormField getFormField( String name )
	{
		return form.getFormField( name );
	}
}
