/*
 *  soapui, copyright (C) 2005 Ole Matzura / eviware.com 
 *
 *  soapUI is free software; you can redistribute it and/or modify it under the 
 *  terms of the GNU Lesser General Public License as published by the Free Software Foundation; 
 *  either version 2.1 of the License, or (at your option) any later version.
 *
 *  soapUI is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
 *  even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 *  See the GNU Lesser General Public License for more details at gnu.org.
 */

package com.eviware.soapui.impl.wsdl.support;

import javax.wsdl.BindingOperation;
import javax.wsdl.Definition;
import javax.wsdl.factory.WSDLFactory;
import javax.wsdl.xml.WSDLReader;

import com.eviware.soapui.impl.wsdl.WsdlInterface;
import com.eviware.soapui.impl.wsdl.WsdlOperation;
import com.eviware.soapui.impl.wsdl.WsdlProject;
import com.eviware.soapui.impl.wsdl.WsdlRequest;
import com.eviware.soapui.impl.wsdl.WsdlSubmitContext;
import com.eviware.soapui.impl.wsdl.support.wsdl.UrlWsdlLoader;
import com.eviware.soapui.impl.wsdl.support.wsdl.WsdlImporter;
import com.eviware.soapui.model.iface.Submit;
import com.eviware.soapui.support.TestCaseWithJetty;

public class WsdlImporterTestCase extends TestCaseWithJetty
{
	public void testOneWayOperationImport() throws Exception
   {
   	WsdlProject project = new WsdlProject();
   	WsdlInterface[] wsdls = WsdlImporter.getInstance().importWsdl( 
   			project, "http://localhost:8082/testonewayop/TestService.wsdl");
   	
   	assertEquals( 1, wsdls.length );
   	
   	WsdlInterface iface = wsdls[0];
   	
   	assertNotNull( iface );
   	assertEquals( 2, iface.getOperationCount() );

   	WsdlOperation operation = (WsdlOperation) iface.getOperationAt( 0 );
   	
   	assertNotNull( operation );
   	assertEquals( "GetDefaultPageData", operation.getName() );
   	
   	Definition definition = readDefinition( "http://localhost:8082/testonewayop/TestService.wsdl" );
   	
   	BindingOperation bindingOperation = operation.findBindingOperation( definition );
   	assertNotNull( bindingOperation );
   	assertEquals( bindingOperation.getName(), operation.getBindingOperationName() );
   	
   	assertNull( operation.getOutputName() );
   	
   	WsdlRequest request = operation.addNewRequest( "TestRequest" );
   	assertNotNull( request );

   	String requestXml = operation.createRequest( true );
   	assertNotNull( requestXml );
   	
   	request.setRequestContent( requestXml );
   	
   	Submit submit = request.submit( new WsdlSubmitContext( null ), false );
   	
   	assertTrue( submit.getResponse().getContentAsString().indexOf( "Error 404 NOT_FOUND" ) > 0 );
   }
   
   public Definition readDefinition( String wsdlUrl ) throws Exception 
   {
      WSDLFactory factory = WSDLFactory.newInstance();
      WSDLReader wsdlReader = factory.newWSDLReader();
      wsdlReader.setFeature("javax.wsdl.verbose", true);
      wsdlReader.setFeature("javax.wsdl.importDocuments", true);
      return wsdlReader.readWSDL( new UrlWsdlLoader( wsdlUrl ) );
   }
}
